@@ -1,6 +1,6 @@
 /*
  * ccid_serial.c: communicate with a GemPC Twin smart card reader
- * Copyright (C) 2001-2003 Ludovic Rousseau <ludovic.rousseau@free.fr>
+ * Copyright (C) 2001-2004 Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
  * Thanks to Niki W. Waibel <niki.waibel@gmx.net> for a prototype version
  * 
@@ -20,7 +20,7 @@
  */
 
 /*
- * $Id: ccid_serial.c,v 1.1.1.3 2004/02/15 13:06:56 rousseau Exp $
+ * $Id: ccid_serial.c,v 1.1.1.5 2004/08/15 20:52:06 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -32,15 +32,19 @@
 #include <errno.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <PCSC/ifdhandler.h>
 
-#include "pcscdefines.h"
+#include "defs.h"
+#include "ccid_ifdhandler.h"
 #include "config.h"
 #include "debug.h"
 #include "ccid.h"
 #include "utils.h"
+#include "commands.h"
 
-/* communication timeout in seconds */
-#define SERIAL_TIMEOUT 2
+/* communication timeout in seconds
+ * the value is set at the end of OpenSerialByName() */
+int SerialTimeout;
 
 #define SYNC 0x03
 #define CTRL_ACK 0x06
@@ -104,7 +108,7 @@ typedef struct
 	/*
 	 * device used ("/dev/ttyS?" under Linux)
 	 */
-	char *device;
+	/*@null@*/ char *device;
 
 	/*
 	 * serial communication buffer
@@ -131,25 +135,33 @@ typedef struct
 /* The _serialDevice structure must be defined before including ccid_serial.h */
 #include "ccid_serial.h"
 
-static _serialDevice serialDevice[PCSCLITE_MAX_READERS] = {
-	[ 0 ... (PCSCLITE_MAX_READERS-1) ] = { -1, NULL }
-};
+/* no need to initialize to 0 since it is static */
+static _serialDevice serialDevice[CCID_DRIVER_MAX_READERS];
+
+/* unexported functions */
+static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
+	int buffer_length, int min_length);
+
+static int get_bytes(unsigned int reader_index, unsigned char *buffer,
+	int length);
+
 
 /*****************************************************************************
  * 
  *				WriteSerial: Send bytes to the card reader
  *
  *****************************************************************************/
-status_t WriteSerial(int lun, int length, unsigned char *buffer)
+status_t WriteSerial(unsigned int reader_index, unsigned int length,
+	unsigned char *buffer)
 {
 	int i;
 	unsigned char lrc;
 	unsigned char low_level_buffer[GEMPCTWIN_MAXBUF];
 
 #ifdef DEBUG_LEVEL_COMM
-	char debug_header[] = "-> 121234 ";
+	char debug_header[] = "-> 123456 ";
 
-	sprintf(debug_header, "-> %06X ", (int)lun);
+	sprintf(debug_header, "-> %06X ", reader_index);
 #endif
 
 	if (length > GEMPCTWIN_MAXBUF-3)
@@ -176,7 +188,7 @@ status_t WriteSerial(int lun, int length, unsigned char *buffer)
 	DEBUG_XXD(debug_header, low_level_buffer, length+3);
 #endif
 
-	if (write(serialDevice[LunToReaderIndex(lun)].fd, low_level_buffer,
+	if (write(serialDevice[reader_index].fd, low_level_buffer,
 		length+3) != length+3)
 	{
 		DEBUG_CRITICAL2("write error: %s", strerror(errno));
@@ -192,7 +204,8 @@ status_t WriteSerial(int lun, int length, unsigned char *buffer)
  *				ReadSerial: Receive bytes from the card reader
  *
  *****************************************************************************/
-status_t ReadSerial(int lun, int *length, unsigned char *buffer)
+status_t ReadSerial(unsigned int reader_index,
+	/*@unused@*/ unsigned int *length, unsigned char *buffer)
 {
 	unsigned char c;
 	int rv;
@@ -205,7 +218,7 @@ status_t ReadSerial(int lun, int *length, unsigned char *buffer)
 
 start:
 	DEBUG_COMM("start");
-	if ((rv = get_bytes(lun, &c, 1)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, &c, 1)) != STATUS_SUCCESS)
 		return rv;
 
 	if (c == RDR_to_PC_NotifySlotChange)
@@ -225,21 +238,27 @@ start:
 
 slot_change:
 	DEBUG_COMM("slot change");
-	if ((rv = get_bytes(lun, &c, 1)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, &c, 1)) != STATUS_SUCCESS)
 		return rv;
 
 	if (c == CARD_ABSENT)
+	{
 		DEBUG_COMM("Card removed");
+	}
 	else
 		if (c == CARD_PRESENT)
+		{
 			DEBUG_COMM("Card inserted");
+		}
 		else
-			DEBUG_COMM2("Unknown card movement: %d", buffer[3]);
+		{
+			DEBUG_COMM2("Unknown card movement: %d", c);
+		}
 	goto start;
 
 sync:
 	DEBUG_COMM("sync");
-	if ((rv = get_bytes(lun, &c, 1)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, &c, 1)) != STATUS_SUCCESS)
 		return rv;
 
 	if (c == CTRL_ACK)
@@ -253,7 +272,7 @@ sync:
 
 nak:
 	DEBUG_COMM("nak");
-	if ((rv = get_bytes(lun, &c, 1)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, &c, 1)) != STATUS_SUCCESS)
 		return rv;
 
 	if (c != (SYNC ^ CTRL_NAK))
@@ -267,14 +286,14 @@ nak:
 ack:
 	DEBUG_COMM("ack");
 	/* normal CCID frame */
-	if ((rv = get_bytes(lun, buffer, 5)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, buffer, 5)) != STATUS_SUCCESS)
 		return rv;
 
 	/* total frame size */
 	to_read = 10+dw2i(buffer, 1);
 
 	DEBUG_COMM2("frame size: %d", to_read);
-	if ((rv = get_bytes(lun, buffer+5, to_read-5)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, buffer+5, to_read-5)) != STATUS_SUCCESS)
 		return rv;
 
 #ifdef DEBUG_LEVEL_COMM
@@ -283,7 +302,7 @@ ack:
 
 	/* lrc */
 	DEBUG_COMM("lrc");
-	if ((rv = get_bytes(lun, &c, 1)) != STATUS_SUCCESS)
+	if ((rv = get_bytes(reader_index, &c, 1)) != STATUS_SUCCESS)
 		return rv;
 
 	DEBUG_COMM2("lrc: 0x%02X", c);
@@ -291,10 +310,7 @@ ack:
 		c ^= buffer[i];
 
 	if (c != (SYNC ^ CTRL_ACK))
-	{
 		DEBUG_CRITICAL2("Wrong LRC: 0x%02X", c);
-		//return STATUS_COMM_ERROR;
-	}
 
 	if (echo)
 	{
@@ -311,10 +327,10 @@ ack:
  *				get_bytes: get n bytes
  *
  *****************************************************************************/
-int get_bytes(int lun, unsigned char *buffer, int length)
+int get_bytes(unsigned int reader_index, unsigned char *buffer, int length)
 {
-	int offset = serialDevice[LunToReaderIndex(lun)].buffer_offset;
-	int offset_last = serialDevice[LunToReaderIndex(lun)].buffer_offset_last;
+	int offset = serialDevice[reader_index].buffer_offset;
+	int offset_last = serialDevice[reader_index].buffer_offset_last;
 
 	DEBUG_COMM3("available: %d, needed: %d", offset_last-offset,
 		length);
@@ -322,8 +338,8 @@ int get_bytes(int lun, unsigned char *buffer, int length)
 	if (offset + length <= offset_last)
 	{
 		DEBUG_COMM("data available");
-		memcpy(buffer, serialDevice[LunToReaderIndex(lun)].buffer + offset, length);
-		serialDevice[LunToReaderIndex(lun)].buffer_offset += length;
+		memcpy(buffer, serialDevice[reader_index].buffer + offset, length);
+		serialDevice[reader_index].buffer_offset += length;
 	}
 	else
 	{
@@ -335,24 +351,25 @@ int get_bytes(int lun, unsigned char *buffer, int length)
 		if (present > 0)
 		{
 			DEBUG_COMM2("some data available: %d", present);
-			memcpy(buffer, serialDevice[LunToReaderIndex(lun)].buffer + offset,
+			memcpy(buffer, serialDevice[reader_index].buffer + offset,
 				present);
 		}
 
 		/* get fresh data */
 		DEBUG_COMM2("get more data: %d", length - present);
-		rv = ReadChunk(lun, serialDevice[LunToReaderIndex(lun)].buffer, sizeof(serialDevice[LunToReaderIndex(lun)].buffer), length - present);
+		rv = ReadChunk(reader_index, serialDevice[reader_index].buffer,
+			sizeof(serialDevice[reader_index].buffer), length - present);
 		if (rv < 0)
 			return STATUS_COMM_ERROR;
 
 		/* fill the buffer */
-		memcpy(buffer + present, serialDevice[LunToReaderIndex(lun)].buffer,
+		memcpy(buffer + present, serialDevice[reader_index].buffer,
 			length - present);
-		serialDevice[LunToReaderIndex(lun)].buffer_offset = length - present;
-		serialDevice[LunToReaderIndex(lun)].buffer_offset_last = rv;
+		serialDevice[reader_index].buffer_offset = length - present;
+		serialDevice[reader_index].buffer_offset_last = rv;
 		DEBUG_COMM3("offset: %d, last_offset: %d",
-			serialDevice[LunToReaderIndex(lun)].buffer_offset,
-			serialDevice[LunToReaderIndex(lun)].buffer_offset_last);
+			serialDevice[reader_index].buffer_offset,
+			serialDevice[reader_index].buffer_offset_last);
 	}
 
 	return STATUS_SUCCESS;
@@ -364,17 +381,20 @@ int get_bytes(int lun, unsigned char *buffer, int length)
  *				ReadChunk: read a minimum number of bytes
  *
  *****************************************************************************/
-int ReadChunk(int lun, unsigned char *buffer, int buffer_length, int min_length)
+static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
+	int buffer_length, int min_length)
 {
-	int fd = serialDevice[LunToReaderIndex(lun)].fd;
+	int fd = serialDevice[reader_index].fd;
+# ifndef S_SPLINT_S
 	fd_set fdset;
+# endif
 	struct timeval t;
 	int i, rv = 0;
 	int already_read;
 #ifdef DEBUG_LEVEL_COMM
-	char debug_header[] = "<- 121234 ";
+	char debug_header[] = "<- 123456 ";
 
-	sprintf(debug_header, "<- %06X ", (int)lun);
+	sprintf(debug_header, "<- %06X ", reader_index);
 #endif
 
 	already_read = 0;
@@ -383,7 +403,7 @@ int ReadChunk(int lun, unsigned char *buffer, int buffer_length, int min_length)
 		/* use select() to, eventually, timeout */
 		FD_ZERO(&fdset);
 		FD_SET(fd, &fdset);
-		t.tv_sec = SERIAL_TIMEOUT;
+		t.tv_sec = SerialTimeout;
 		t.tv_usec = 0;
 
 		i = select(fd+1, &fdset, NULL, NULL, &t);
@@ -395,7 +415,7 @@ int ReadChunk(int lun, unsigned char *buffer, int buffer_length, int min_length)
 		else
 			if (i == 0)
 			{
-				DEBUG_COMM2("Timeout! (%d sec)", SERIAL_TIMEOUT);
+				DEBUG_COMM2("Timeout! (%d sec)", SerialTimeout);
 				return -1;
 			}
 
@@ -424,11 +444,11 @@ int ReadChunk(int lun, unsigned char *buffer, int buffer_length, int min_length)
  *				OpenSerial: open the port
  *
  *****************************************************************************/
-status_t OpenSerial(int lun, int channel)
+status_t OpenSerial(unsigned int reader_index, int channel)
 {
 	char dev_name[FILENAME_MAX];
 
-	DEBUG_COMM3("Lun: %X, Channel: %d", lun, channel);
+	DEBUG_COMM3("Reader index: %X, Channel: %d", reader_index, channel);
 
 	/*
 	 * Conversion of old-style ifd-hanler 1.0 CHANNELID 
@@ -453,7 +473,7 @@ status_t OpenSerial(int lun, int channel)
 
 	sprintf(dev_name, "/dev/pcsc/%d", (int) channel);
 
-	return OpenSerialByName(lun, dev_name);
+	return OpenSerialByName(reader_index, dev_name);
 } /* OpenSerial */
 
 /*****************************************************************************
@@ -461,16 +481,16 @@ status_t OpenSerial(int lun, int channel)
  *				OpenSerialByName: open the port
  *
  *****************************************************************************/
-status_t OpenSerialByName(int lun, char *dev_name)
+status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 {
 	struct termios current_termios;
 	int i;
-	int reader = LunToReaderIndex(lun);
+	unsigned int reader = reader_index;
 
-	DEBUG_COMM3("Lun: %X, Device: %d", lun, dev_name);
+	DEBUG_COMM3("Reader index: %X, Device: %d", reader_index, dev_name);
 
 	/* check if the same channel is not already used */
-	for (i=0; i<PCSCLITE_MAX_READERS; i++)
+	for (i=0; i<CCID_DRIVER_MAX_READERS; i++)
 	{
 		if (serialDevice[i].device &&
 			strcmp(serialDevice[i].device, dev_name) == 0)
@@ -482,7 +502,7 @@ status_t OpenSerialByName(int lun, char *dev_name)
 
 	serialDevice[reader].fd = open(dev_name, O_RDWR | O_NOCTTY);
 
-	if (serialDevice[reader].fd <= 0)
+	if (-1 == serialDevice[reader].fd)
 	{
 		DEBUG_CRITICAL3("open %s: %s", dev_name, strerror(errno));
 		return STATUS_UNSUCCESSFUL;
@@ -532,17 +552,45 @@ status_t OpenSerialByName(int lun, char *dev_name)
 		return STATUS_UNSUCCESSFUL;
 	}
 
-	serialDevice[reader].ccid.bSeq = 0;
+	serialDevice[reader].ccid.real_bSeq = 0;
+	serialDevice[reader].ccid.pbSeq = &serialDevice[reader].ccid.real_bSeq;
 	serialDevice[reader].ccid.readerID = GEMPCTWIN;
 	serialDevice[reader].ccid.dwMaxCCIDMessageLength = 271;
 	serialDevice[reader].ccid.dwMaxIFSD = 254;
 	serialDevice[reader].ccid.dwFeatures = 0x00010230;
+	serialDevice[reader].ccid.bPINSupport = 0x0;
 	serialDevice[reader].ccid.dwDefaultClock = 4000;
 	serialDevice[reader].ccid.dwMaxDataRate = 344086;
+	serialDevice[reader].ccid.bMaxSlotIndex = 0;
+	serialDevice[reader].ccid.bCurrentSlotIndex = 0;
 
 	serialDevice[reader].buffer_offset = 0;
 	serialDevice[reader].buffer_offset_last = 0;
 
+	/* perform a command to be sure a GemPC Twin reader is connected
+	 * get the reader firmware */
+	{
+		unsigned char tx_buffer[] = "\x02";
+		unsigned char rx_buffer[50];
+		unsigned int rx_length = sizeof(rx_buffer);
+
+		/* 2 seconds timeout to not wait too long if no reader is connected */
+		SerialTimeout = 2;
+
+		if (IFD_SUCCESS != CmdEscape(reader_index, tx_buffer, sizeof(tx_buffer),
+			rx_buffer, &rx_length))
+		{
+			DEBUG_CRITICAL("Get firmware failed. Maybe the reader is not connected");
+			return STATUS_UNSUCCESSFUL;
+		}
+
+		/* normal timeout: 1 minute to allow long time APDU */
+		SerialTimeout = 60;
+
+		rx_buffer[rx_length] = '\0';
+		DEBUG_INFO2("Firmware: %s", rx_buffer);
+	}
+
 	return STATUS_SUCCESS;
 } /* OpenSerialByName */
 
@@ -552,9 +600,9 @@ status_t OpenSerialByName(int lun, char *dev_name)
  *				CloseSerial: close the port
  *
  *****************************************************************************/
-status_t CloseSerial(int lun)
+status_t CloseSerial(unsigned int reader_index)
 {
-	int reader = LunToReaderIndex(lun);
+	unsigned int reader = reader_index;
 
 	close(serialDevice[reader].fd);
 	serialDevice[reader].fd = -1;
@@ -571,9 +619,9 @@ status_t CloseSerial(int lun)
  *					get_ccid_descriptor
  *
  ****************************************************************************/
-_ccid_descriptor *get_ccid_descriptor(int lun)
+_ccid_descriptor *get_ccid_descriptor(unsigned int reader_index)
 {
-	return &serialDevice[LunToReaderIndex(lun)].ccid;
+	return &serialDevice[reader_index].ccid;
 } /* get_ccid_descriptor */
 
 
