@@ -18,14 +18,18 @@
 */
 
 /*
- * $Id: ccid_serial.h,v 1.1.1.3 2004/02/15 13:06:56 rousseau Exp $
+ * $Id: ccid_serial.h,v 1.1.1.5 2004/08/15 20:52:06 rousseau Exp $
  */
 
-status_t OpenSerial(int lun, int channel);
-status_t OpenSerialByName(int lun, char *dev_name);
-status_t WriteSerial(int lun, int length, unsigned char *Buffer);
-status_t ReadSerial(int lun, int *length, unsigned char *Buffer);
-int get_bytes(int lun, unsigned char *buffer, int length);
-int ReadChunk(int lun, unsigned char *buffer, int buffer_length, int min_length);
-status_t CloseSerial(int lun);
+status_t OpenSerial(unsigned int reader_index, int channel);
+
+status_t OpenSerialByName(unsigned int reader_index, char *dev_name);
+
+status_t WriteSerial(unsigned int reader_index, unsigned int length,
+	unsigned char *Buffer);
+
+status_t ReadSerial(unsigned int reader_index, unsigned int *length,
+	unsigned char *Buffer);
+
+status_t CloseSerial(unsigned int reader_index);
 
