@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c,v 1.18 2004/02/12 16:09:32 rousseau Exp $
+ * $Id: ccid_usb.c,v 1.39 2004/08/06 11:18:41 rousseau Exp $
  */
 
 #define __CCID_USB__
@@ -26,14 +26,18 @@
 #include <stdio.h>
 #include <string.h> 
 #include <errno.h>
+# ifdef S_SPLINT_S
+# include <sys/types.h>
+# endif
 #include <usb.h>
 
-#include "pcscdefines.h"
+#include "ccid.h"
 #include "config.h"
 #include "debug.h"
+#include "defs.h"
 #include "utils.h"
 #include "parser.h"
-#include "ccid.h"
+#include "ccid_ifdhandler.h"
 
 
 /* read timeout
@@ -46,6 +50,13 @@
  * we don't have to wait a long time since the card was doing nothing */
 #define USB_WRITE_TIMEOUT (5 * 1000)  /* 5 seconds timeout */
 
+/*
+ * Proprietary USB Class (0xFF) are (or are not) accepted
+ * A proprietary class is used for devices released before the final CCID
+ * specifications were ready.
+ * We should not have problems with non CCID devices becasue the
+ * Manufacturer and Product ID are also used to identify the device */
+#define ALLOW_PROPRIETARY_CLASS
 
 #define BUS_DEVICE_STRSIZE 32
 
@@ -70,9 +81,10 @@ typedef struct
 /* The _usbDevice structure must be defined before including ccid_usb.h */
 #include "ccid_usb.h"
 
-static _usbDevice usbDevice[PCSCLITE_MAX_READERS] = {
-	[ 0 ... (PCSCLITE_MAX_READERS-1) ] = { NULL, NULL, 0, 0 }
-};
+static int get_end_points(struct usb_device *dev, _usbDevice *usb_device);
+
+/* ne need to initialize to 0 since it is static */
+static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
 
 #define PCSCLITE_MANUKEY_NAME                   "ifdVendorID"
 #define PCSCLITE_PRODKEY_NAME                   "ifdProductID"
@@ -84,9 +96,9 @@ static _usbDevice usbDevice[PCSCLITE_MAX_READERS] = {
  *					OpenUSB
  *
  ****************************************************************************/
-status_t OpenUSB(int lun, int Channel)
+status_t OpenUSB(unsigned int reader_index, /*@unused@*/ int Channel)
 {
-	return OpenUSBByName(lun, NULL);
+	return OpenUSBByName(reader_index, NULL);
 } /* OpenUSB */
 
 
@@ -95,23 +107,25 @@ status_t OpenUSB(int lun, int Channel)
  *					OpenUSBByName
  *
  ****************************************************************************/
-status_t OpenUSBByName(int lun, char *device)
+status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 {
 	static struct usb_bus *busses = NULL;
-	int reader = LunToReaderIndex(lun);
 	int alias = 0;
 	struct usb_bus *bus;
 	struct usb_dev_handle *dev_handle;
 	char keyValue[TOKEN_MAX_VALUE_SIZE];
 	int vendorID, productID;
 	char infofile[FILENAME_MAX];
-	int device_vendor, device_product;
+	unsigned int device_vendor, device_product;
+	char *dirname = NULL, *filename = NULL;
+	static int previous_reader_index = -1;
 
-	DEBUG_COMM3("Lun: %X, Device: %s", lun, device);
+	DEBUG_COMM3("Reader index: %X, Device: %s", reader_index, device);
 
 	/* device name specified */
 	if (device)
 	{
+		/* format: usb:%04x/%04x, vendor, product */
 		if (strncmp("usb:", device, 4) != 0)
 		{
 			DEBUG_CRITICAL2("device name does not start with \"usb:\": %s",
@@ -124,6 +138,33 @@ status_t OpenUSBByName(int lun, char *device)
 			DEBUG_CRITICAL2("device name can't be parsed: %s", device);
 			return STATUS_UNSUCCESSFUL;
 		}
+
+		/* format usb:%04x/%04x:libusb:%s
+		 * with %s set to %s:%s, dirname, filename */
+		if ((dirname = strstr(device, "libusb:")) != NULL)
+		{
+			/* dirname points to the first char after libusb: */
+			dirname += strlen("libusb:");
+
+			/* search the : (separation) char */
+			filename = strchr(dirname, ':');
+
+			if (filename)
+			{
+				/* end the dirname string */
+				*filename = '\0';
+
+				/* filename points to the first char after : */
+				filename++;
+			}
+			else
+			{
+				/* parse failed */
+				dirname = NULL;
+
+				DEBUG_CRITICAL2("can't parse using libusb scheme: %s", device);
+			}
+		}
 	}
 
 	if (busses == NULL)
@@ -140,10 +181,11 @@ status_t OpenUSBByName(int lun, char *device)
 		return STATUS_UNSUCCESSFUL;
 	}
 
-	/* is the lun already used? */
-	if (usbDevice[reader].handle != NULL)
+	/* is the reader_index already used? */
+	if (usbDevice[reader_index].handle != NULL)
 	{
-		DEBUG_CRITICAL2("USB driver with lun %X already in use", lun);
+		DEBUG_CRITICAL2("USB driver with index %X already in use",
+			reader_index);
 		return STATUS_UNSUCCESSFUL;
 	}
 
@@ -153,7 +195,9 @@ status_t OpenUSBByName(int lun, char *device)
 
 	/* general driver info */
 	if (!LTPBundleFindValueWithKey(infofile, "ifdManufacturerString", keyValue, 0))
+	{
 		DEBUG_CRITICAL2("Manufacturer: %s", keyValue);
+	}
 	else
 	{
 		DEBUG_CRITICAL2("LTPBundleFindValueWithKey error. Can't find %s?",
@@ -161,11 +205,15 @@ status_t OpenUSBByName(int lun, char *device)
 		return STATUS_UNSUCCESSFUL;
 	}
 	if (!LTPBundleFindValueWithKey(infofile, "ifdProductString", keyValue, 0))
+	{
 		DEBUG_CRITICAL2("ProductString: %s", keyValue);
+	}
 	else
 		return STATUS_UNSUCCESSFUL;
 	if (!LTPBundleFindValueWithKey(infofile, "Copyright", keyValue, 0))
+	{
 		DEBUG_CRITICAL2("Copyright: %s", keyValue);
+	}
 	else
 		return STATUS_UNSUCCESSFUL;
 	vendorID = strlen(keyValue);
@@ -176,11 +224,11 @@ status_t OpenUSBByName(int lun, char *device)
 	/* for any supported reader */
 	while (LTPBundleFindValueWithKey(infofile, PCSCLITE_MANUKEY_NAME, keyValue, alias) == 0)
 	{
-		vendorID = strtoul(keyValue, 0, 16);
+		vendorID = strtoul(keyValue, NULL, 0);
 
 		if (LTPBundleFindValueWithKey(infofile, PCSCLITE_PRODKEY_NAME, keyValue, alias))
 			goto end;
-		productID = strtoul(keyValue, 0, 16);
+		productID = strtoul(keyValue, NULL, 0);
 
 		if (LTPBundleFindValueWithKey(infofile, PCSCLITE_NAMEKEY_NAME, keyValue, alias))
 			goto end;
@@ -201,6 +249,11 @@ status_t OpenUSBByName(int lun, char *device)
 			/* any device on this bus */
 			for (dev = bus->devices; dev; dev = dev->next)
 			{
+				/* device defined by name? */
+				if (dirname && (strcmp(dirname, bus->dirname)
+					|| strcmp(filename, dev->filename)))
+					continue;
+
 				if (dev->descriptor.idVendor == vendorID
 					&& dev->descriptor.idProduct == productID)
 				{
@@ -211,7 +264,7 @@ status_t OpenUSBByName(int lun, char *device)
 					/* is it already opened? */
 					already_used = FALSE;
 
-					for (r=0; r<PCSCLITE_MAX_READERS; r++)
+					for (r=0; r<CCID_DRIVER_MAX_READERS; r++)
 					{
 						if (usbDevice[r].dev)
 						{
@@ -226,8 +279,26 @@ status_t OpenUSBByName(int lun, char *device)
 					/* this reader is already managed by us */
 					if (already_used)
 					{
-						DEBUG_INFO3("USB device %s/%s already in use. Checking next one.",
-							bus->dirname, dev->filename);
+						if ((previous_reader_index != -1)
+							&& (strcmp(usbDevice[previous_reader_index].dev->bus->dirname, bus->dirname)  == 0)
+							&& (strcmp(usbDevice[previous_reader_index].dev->filename, dev->filename) == 0)
+							&& usbDevice[previous_reader_index].ccid.bCurrentSlotIndex < usbDevice[previous_reader_index].ccid.bMaxSlotIndex)
+						{
+							/* we reuse the same device
+							 * and the reader is multi-slot */
+							usbDevice[reader_index] = usbDevice[previous_reader_index];
+							usbDevice[reader_index].ccid.pbSeq = usbDevice[previous_reader_index].ccid.pbSeq;
+							usbDevice[reader_index].ccid.bCurrentSlotIndex++;
+							DEBUG_INFO2("Opening slot: %d",
+								usbDevice[reader_index].ccid.bCurrentSlotIndex);
+							goto end;
+						}
+						else
+						{
+							DEBUG_INFO3("USB device %s/%s already in use."
+								" Checking next one.",
+								bus->dirname, dev->filename);
+						}
 
 						continue;
 					}
@@ -281,32 +352,39 @@ status_t OpenUSBByName(int lun, char *device)
 						 bus->dirname, dev->filename);
 
 					/* Get Endpoints values*/
-					get_end_points(dev, &usbDevice[reader]);
+					get_end_points(dev, &usbDevice[reader_index]);
 
 					/* store device information */
-					usbDevice[reader].handle = dev_handle;
-					usbDevice[reader].dev = dev;
+					usbDevice[reader_index].handle = dev_handle;
+					usbDevice[reader_index].dev = dev;
 
 					/* CCID common informations */
-					usbDevice[reader].ccid.bSeq = 0;
-					usbDevice[reader].ccid.readerID =
+					usbDevice[reader_index].ccid.real_bSeq = 0;
+					usbDevice[reader_index].ccid.pbSeq = &usbDevice[reader_index].ccid.real_bSeq;
+					usbDevice[reader_index].ccid.readerID =
 						(dev->descriptor.idVendor << 16) +
 						dev->descriptor.idProduct;
-					usbDevice[reader].ccid.dwFeatures = dw2i(usb_interface->altsetting->extra, 40);
-					usbDevice[reader].ccid.dwMaxCCIDMessageLength = dw2i(usb_interface->altsetting->extra, 44);
-					usbDevice[reader].ccid.dwMaxIFSD = dw2i(usb_interface->altsetting->extra, 28);
-					usbDevice[reader].ccid.dwDefaultClock = dw2i(usb_interface->altsetting->extra, 10);
-					usbDevice[reader].ccid.dwMaxDataRate = dw2i(usb_interface->altsetting->extra, 23);
-
+					usbDevice[reader_index].ccid.dwFeatures = dw2i(usb_interface->altsetting->extra, 40);
+					usbDevice[reader_index].ccid.bPINSupport = usb_interface->altsetting->extra[52];
+					usbDevice[reader_index].ccid.dwMaxCCIDMessageLength = dw2i(usb_interface->altsetting->extra, 44);
+					usbDevice[reader_index].ccid.dwMaxIFSD = dw2i(usb_interface->altsetting->extra, 28);
+					usbDevice[reader_index].ccid.dwDefaultClock = dw2i(usb_interface->altsetting->extra, 10);
+					usbDevice[reader_index].ccid.dwMaxDataRate = dw2i(usb_interface->altsetting->extra, 23);
+					usbDevice[reader_index].ccid.bMaxSlotIndex = usb_interface->altsetting->extra[4];
+					usbDevice[reader_index].ccid.bCurrentSlotIndex = 0;
 					goto end;
 				}
 			}
 		}
 	}
 end:
-	if (usbDevice[reader].handle == NULL)
+	if (usbDevice[reader_index].handle == NULL)
 		return STATUS_UNSUCCESSFUL;
 
+	/* memorise the current reader_index so we can detect
+	 * a new OpenUSBByName on a multi slot reader */
+	previous_reader_index = reader_index;
+
 	return STATUS_SUCCESS;
 } /* OpenUSBByName */
 
@@ -316,27 +394,29 @@ end:
  *					WriteUSB
  *
  ****************************************************************************/
-status_t WriteUSB(int lun, int length, unsigned char *buffer)
+status_t WriteUSB(unsigned int reader_index, unsigned int length,
+	unsigned char *buffer)
 {
 	int rv;
-	int reader = LunToReaderIndex(lun);
 #ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "-> 121234 ";
 
-	sprintf(debug_header, "-> %06X ", (int)lun);
+	sprintf(debug_header, "-> %06X ", (int)reader_index);
 #endif
 
 #ifdef DEBUG_LEVEL_COMM
 	DEBUG_XXD(debug_header, buffer, length);
 #endif
 
-	rv = usb_bulk_write(usbDevice[reader].handle, usbDevice[reader].bulk_out, buffer, length, USB_WRITE_TIMEOUT);
+	rv = usb_bulk_write(usbDevice[reader_index].handle,
+		usbDevice[reader_index].bulk_out, (char *)buffer, length,
+		USB_WRITE_TIMEOUT);
 
 	if (rv < 0)
 	{
 		DEBUG_CRITICAL4("usb_bulk_write(%s/%s): %s",
-			usbDevice[reader].dev->bus->dirname,
-			usbDevice[reader].dev->filename, strerror(errno));
+			usbDevice[reader_index].dev->bus->dirname,
+			usbDevice[reader_index].dev->filename, strerror(errno));
 		return STATUS_UNSUCCESSFUL;
 	}
 
@@ -349,28 +429,31 @@ status_t WriteUSB(int lun, int length, unsigned char *buffer)
  *					ReadUSB
  *
  ****************************************************************************/
-status_t ReadUSB(int lun, int * length, unsigned char *buffer)
+status_t ReadUSB(unsigned int reader_index, unsigned int * length,
+	unsigned char *buffer)
 {
 	int rv;
-	int reader = LunToReaderIndex(lun);
 #ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "<- 121234 ";
 
-	sprintf(debug_header, "<- %06X ", (int)lun);
+	sprintf(debug_header, "<- %06X ", (int)reader_index);
 #endif
 
-
-	rv = usb_bulk_read(usbDevice[reader].handle, usbDevice[reader].bulk_in, buffer, *length, USB_READ_TIMEOUT);
-	*length = rv;
+	rv = usb_bulk_read(usbDevice[reader_index].handle,
+		usbDevice[reader_index].bulk_in, (char *)buffer, *length,
+		USB_READ_TIMEOUT);
 
 	if (rv < 0)
 	{
+		*length = 0;
 		DEBUG_CRITICAL4("usb_bulk_read(%s/%s): %s",
-			usbDevice[reader].dev->bus->dirname,
-			usbDevice[reader].dev->filename, strerror(errno));
+			usbDevice[reader_index].dev->bus->dirname,
+			usbDevice[reader_index].dev->filename, strerror(errno));
 		return STATUS_UNSUCCESSFUL;
 	}
 
+	*length = rv;
+
 #ifdef DEBUG_LEVEL_COMM
 	DEBUG_XXD(debug_header, buffer, *length);
 #endif
@@ -384,32 +467,33 @@ status_t ReadUSB(int lun, int * length, unsigned char *buffer)
  *					CloseUSB
  *
  ****************************************************************************/
-status_t CloseUSB(int lun)
+status_t CloseUSB(unsigned int reader_index)
 {
 	struct usb_interface *usb_interface;
 	int interface;
-	int reader = LunToReaderIndex(lun);
 
 	/* device not opened */
-	if (usbDevice[reader].dev == NULL)
+	if (usbDevice[reader_index].dev == NULL)
 		return STATUS_UNSUCCESSFUL;
 
 	DEBUG_COMM3("Closing USB device: %s/%s",
-		usbDevice[reader].dev->bus->dirname, usbDevice[reader].dev->filename);
+		usbDevice[reader_index].dev->bus->dirname,
+		usbDevice[reader_index].dev->filename);
 
-  	usb_interface = get_ccid_usb_interface(usbDevice[reader].dev);
+  	usb_interface = get_ccid_usb_interface(usbDevice[reader_index].dev);
 	interface = usb_interface ?
 		usb_interface->altsetting->bInterfaceNumber :
-		usbDevice[reader].dev->config->interface->altsetting->bInterfaceNumber;
+		usbDevice[reader_index].dev->config->interface->altsetting->bInterfaceNumber;
 	
-	usb_release_interface(usbDevice[reader].handle, interface);
+	/* reset so that bSeq starts at 0 again */
+	usb_reset(usbDevice[reader_index].handle);
 
-	usb_reset(usbDevice[reader].handle);
-	usb_close(usbDevice[reader].handle);
+	usb_release_interface(usbDevice[reader_index].handle, interface);
+	usb_close(usbDevice[reader_index].handle);
 
 	/* mark the resource unused */
-	usbDevice[reader].handle = NULL;
-	usbDevice[reader].dev = NULL;
+	usbDevice[reader_index].handle = NULL;
+	usbDevice[reader_index].dev = NULL;
 
 	return STATUS_SUCCESS;
 } /* CloseUSB */
@@ -420,9 +504,9 @@ status_t CloseUSB(int lun)
  *					get_ccid_descriptor
  *
  ****************************************************************************/
-_ccid_descriptor *get_ccid_descriptor(int lun)
+_ccid_descriptor *get_ccid_descriptor(unsigned int reader_index)
 {
-	return &usbDevice[LunToReaderIndex(lun)].ccid;
+	return &usbDevice[reader_index].ccid;
 } /* get_ccid_descriptor */
 
 
@@ -434,7 +518,7 @@ _ccid_descriptor *get_ccid_descriptor(int lun)
 int get_desc(int channel, usb_dev_handle **handle, struct
 	usb_device **dev)
 {
-	if (channel < 0 || channel > PCSCLITE_MAX_READERS)
+	if (channel < 0 || channel > CCID_DRIVER_MAX_READERS)
 		return 1;
 
 	*handle = usbDevice[channel].handle;
@@ -449,7 +533,7 @@ int get_desc(int channel, usb_dev_handle **handle, struct
  *					get_end_points
  *
  ****************************************************************************/
-int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
+static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
 {
 	int i;
 	int bEndpointAddress;
@@ -481,7 +565,7 @@ int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
  *					get_ccid_usb_interface
  *
  ****************************************************************************/
-struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
+/*@null@*/ struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
 {
 	struct usb_interface *usb_interface = NULL; 
 
@@ -491,7 +575,12 @@ struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
 		int ii;
 		for (ii=0; ii<dev->config->bNumInterfaces; ii++)
 		{
-			if (dev->config->interface[ii].altsetting->bInterfaceClass == 0xb)
+			/* CCID Class? */
+			if (dev->config->interface[ii].altsetting->bInterfaceClass == 0xb
+#ifdef ALLOW_PROPRIETARY_CLASS
+				|| dev->config->interface[ii].altsetting->bInterfaceClass == 0xff
+#endif
+				)
 			{
 				usb_interface = &dev->config->interface[ii];
 				break;
