@@ -18,20 +18,24 @@
 */
 
 /*
- * $Id: ccid_usb.h,v 1.5 2004/01/29 10:32:20 rousseau Exp $
+ * $Id: ccid_usb.h,v 1.11 2004/07/28 08:33:18 rousseau Exp $
  */
 
-status_t OpenUSB(int lun, int channel);
-status_t OpenUSBByName(int lun, char *device);
-status_t WriteUSB(int lun, int length, unsigned char *Buffer);
-status_t ReadUSB(int lun, int *length, unsigned char *Buffer);
-status_t CloseUSB(int lun);
+status_t OpenUSB(unsigned int reader_index, int channel);
+
+status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device);
+
+status_t WriteUSB(unsigned int reader_index, unsigned int length,
+	unsigned char *Buffer);
+
+status_t ReadUSB(unsigned int reader_index, unsigned int *length,
+	unsigned char *Buffer);
+
+status_t CloseUSB(unsigned int reader_index);
 
 #ifdef __USB_H__
 int get_desc(int channel, usb_dev_handle **handle, struct usb_device **dev);
-#endif
-#ifdef __CCID_USB__
-int get_end_points(struct usb_device *dev, _usbDevice *usb_device);
-struct usb_interface *get_ccid_usb_interface(struct usb_device *dev);
+
+/*@null@*/ struct usb_interface *get_ccid_usb_interface(struct usb_device *dev);
 #endif
 
