@@ -18,11 +18,12 @@
 */
 
 /*
- * $Id: defs.h,v 1.4 2004/01/27 13:32:55 rousseau Exp $
+ * $Id: defs.h,v 1.7 2004/06/30 13:30:04 rousseau Exp $
  */
 
-#include "pcscdefines.h"
-#include "protocol_t1/protocol_t1.h"
+#include <PCSC/pcsclite.h>
+
+#include "openct/proto-t1.h"
 
 typedef struct CCID_DESC
 {
@@ -40,15 +41,22 @@ typedef struct CCID_DESC
 	/*
 	 * T=1 Protocol context
 	 */
-	Protocol_T1 t1;
+	t1_state_t t1;
 } CcidDesc;
 
+typedef enum {
+	STATUS_SUCCESS               = 0xFA,
+	STATUS_UNSUCCESSFUL          = 0xFB,
+	STATUS_COMM_ERROR            = 0xFC,
+	STATUS_DEVICE_PROTOCOL_ERROR = 0xFD
+} status_t;
+
 /* Powerflag (used to detect quick insertion removals unnoticed by the 
  * resource manager)Initial value */
 #define POWERFLAGS_RAZ 0x00
 /* Flag set when a power up has been requested */
 #define MASK_POWERFLAGS_PUP 0x01
-/* Flag set when a power down is requestedA */
+/* Flag set when a power down is requested */
 #define MASK_POWERFLAGS_PDWN 0x02
 
 /* Communication buffer size (max=cmd+adpu+Lc+data+Le) */
