@@ -1,6 +1,6 @@
 /*
     utils.c:
-    Copyright (C) 2003   Ludovic Rousseau
+    Copyright (C) 2003-2004   Ludovic Rousseau
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,15 +18,16 @@
 */
 
 /*
- * $Id: utils.h,v 1.1.1.1 2003/08/12 12:40:24 rousseau Exp $
+ * $Id: utils.h,v 1.2 2004/07/28 08:30:30 rousseau Exp $
  */
 
-#define LunToReaderIndex(Lun) (Lun>>16)
-
 #ifndef TRUE
 #define FALSE 0
 #define TRUE 1
 #endif
 
-int CheckLun(DWORD Lun);
+void InitReaderIndex(void);
+int GetNewReaderIndex(const DWORD Lun);
+int LunToReaderIndex(DWORD Lun);
+int ReleaseReaderIndex(const int index);
 
